#ifndef CONF_H
#define CONF_H
/*
 * Copyright (C) 2009 Väinö Helminen <vah78@yahoo.com>
 *
 * $Id: conf.h,v 1.1 2009-04-18 05:40:14 vah Exp $
 */

#include <X11/Xlib.h>

typedef enum { TERM=0, KEY, COMMAND } Type;


typedef union Map {

  struct Common {
    unsigned eventcode:16; // undefined if type == TERM
    unsigned type:8;
    unsigned state:8; // undefined if type == TERM
  } common;

  struct Key {
    unsigned eventcode:16;
    unsigned type:8;
    unsigned state:8;

    unsigned keycode:16;
    unsigned modifiers:8;
    unsigned repeat:8;
  } key;

  struct Command {
    unsigned eventcode:16;
    unsigned type:8;
    unsigned state:8;

    unsigned command:16;
  } command;

} Mapping;


typedef struct Data {
  int devices;
  int *fd;
  Mapping **mapping;
  const char **command;
  Display *display;
  int verbose; // boolean flag for verbose mode
} Data;


/*
 * Parse configuration file.
 *
 * Parameters:
 *   filename - filename of the configuration file
 *   data     - pointer to a Data struct
 *
 * Returns:
 *    0 - OK
 *   -1 - error
 *
 * data must have following members initialized before calling this:
 *   display - need connection to X server for converting keysym 
 *             strings to keycodes
 *   verbose - verbose mode on/off
 * parseConf will initialize and set the rest of the members.
 */
int parseConf(const char *filename, Data *data);

#endif
