/*
 * Copyright (C) 2009 Väinö Helminen <vah78@yahoo.com>
 *
 * $Id: event.c,v 1.1.1.1 2009-03-13 05:24:33 vah Exp $
 */
#include <unistd.h>
#include <linux/input.h>
#include "event.h"

int readEvents(int *fd, int n, KeyCallback callback, void *datap)
{
  // sanity check
  if(!fd || n < 1 || !callback) return -2;

  while(1) {
    // prepare fd set
    fd_set readset;
    FD_ZERO(&readset);
    for(int i = 0; i < n; ++i) {
      FD_SET(fd[i], &readset);
    }

    // wait for something to read
    if((select(fd[n-1]+1, &readset, NULL, NULL, NULL)) < 0) {
      return -1;
    }

    // read all events that are available
    for(int i = 0; i < n; ++i) {
      if(!FD_ISSET(fd[i], &readset)) {
	continue;
      }

      struct input_event ev[64];
      int rd = read(fd[i], ev, sizeof ev);
      rd /= (int)sizeof *ev;
      if(rd < 1) return -1; // error reading

      // call the callback function for each keyboard event and ignore other events
      for(int j = 0; j < rd; ++j) {
	if(ev[j].type == EV_KEY) {
	  if(callback(datap, i, ev[j].code, ev[j].value))
	    return 0;
	}
      }
    }
  }
  return 0;
}
